#! /usr/bin/env python3
# python3 setup.py --version
# FOR RELEASE:
# manually update exotic/version.py
# git add exotic/version.py && git commit -m "#<ticket>:Updated version for release" && git push
# release/tag
# git checkout exotic/version.py && python3 setup.py sdist bdist_wheel --universal  # upload
# pip3 install twine  # https://twine.readthedocs.io/en/latest/
# twine check dist/* && twine upload dist/*
# LOCAL TESTING:
# pip3 install --exists-action w --progress-bar ascii --use-feature=2020-resolver -r requirements.txt
# pip3 install exotic --no-index --find-links file:///proj/survey-ws/source/EXOTIC/dist/
# rm -r dist && pip3 uninstall exotic

import re
import time
from pathlib import Path

import setuptools

# Package meta-data sane defaults
AUTHOR = "Exoplanet Watch at NASA JPL"
AUTHOR_EMAIL = "exoplanetwatch@jpl.nasa.gov"
DESCRIPTION = "EXOTIC: EXOplanet Transit Interpretation Code"
NAME = "exotic"
PYTHON_REQUIREMENTS = ">=3.7.0"
URL = "https://github.com/rzellem/EXOTIC"

REQUIREMENTS_SETUP = ['setuptools_scm',
                      'importlib-metadata ~= 1.7 ; python_version >= "3.7"']


def description_read():
    description_long = ""
    description_path = Path("README.md")
    if not description_path.exists():
        return description_long
    with description_path.open('r') as f:
        description_long = f.read()
    return description_long


def license_read():
    annum = time.gmtime().tm_year
    # provide one-line summary per LICENSE spec
    lic = f"Proprietary -- Copyright (c) 2019-{annum}, California Institute of Technology." 
    return lic


def requirements_read():
    requirements = []
    requirements_path = Path("requirements.txt")
    if not requirements_path.exists():
        return requirements
    with requirements_path.open('r') as f:
        for line in f:
            # detect and skip comment lines
            if re.match(r"^\s*#.*", line):
                continue
            requirements.append(str(line).strip())
    return requirements


setuptools.setup(name=NAME,
                 use_scm_version={
                     'write_to': 'exotic/version.py',
                     'write_to_template': '__version__ = "{version}"'
                 },
                 description=DESCRIPTION,
                 long_description=description_read(),
                 long_description_content_type='text/markdown',
                 url=URL,
                 author=AUTHOR,
                 author_email=AUTHOR_EMAIL,
                 license=license_read(),
                 classifiers=[
                     # Trove classifiers
                     # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
                     'Development Status :: 4 - Beta',
                     'Intended Audience :: End Users/Desktop',
                     'Intended Audience :: Science/Research',
                     'License :: Other/Proprietary License',
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 3',
                     'Programming Language :: Python :: 3.7',
                     'Programming Language :: Python :: Implementation :: CPython',
                     'Programming Language :: Python :: Implementation :: PyPy',
                     'Topic :: Scientific/Engineering :: Astronomy'
                 ],
                 keywords='nasa jpl exoplanet transit citizen science astronomy bayesian nested-sampler',
                 project_urls={
                    "Documentation": "https://github.com/rzellem/EXOTIC/wiki",
                    "Site": "https://exoplanets.nasa.gov/exoplanet-watch",
                    "Source": "https://github.com/rzellem/EXOTIC",
                    "Tracker": "https://github.com/rzellem/EXOTIC/issues"
                 },
                 #  https://setuptools.readthedocs.io/en/latest/setuptools.html#including-data-files
                 packages=setuptools.find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
                 include_package_data=True,
                 zip_safe=False,
                 install_requires=requirements_read(),
                 python_requires=PYTHON_REQUIREMENTS,
                 setup_requires=REQUIREMENTS_SETUP,
                 entry_points={
                     'console_scripts': [
                         'exotic = exotic.exotic:main',
                     ],
                 }
                 )



# ########################################################################### #
#    Copyright (c) 2019-2020, California Institute of Technology.
#    All rights reserved.  Based on Government Sponsored Research under
#    contracts NNN12AA01C, NAS7-1407 and/or NAS7-03001.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#      1. Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#      2. Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer in
#         the documentation and/or other materials provided with the
#         distribution.
#      3. Neither the name of the California Institute of
#         Technology (Caltech), its operating division the Jet Propulsion
#         Laboratory (JPL), the National Aeronautics and Space
#         Administration (NASA), nor the names of its contributors may be
#         used to endorse or promote products derived from this software
#         without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CALIFORNIA
#    INSTITUTE OF TECHNOLOGY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#    TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ########################################################################### #
#    EXOplanet Transit Interpretation Code (EXOTIC)
#    # NOTE: See companion file version.py for version info.
# ########################################################################### #
import codecs
from pathlib import Path


def read_relative(rel_path):
    here = Path(__file__).parent
    with codecs.open(here / rel_path, 'r') as fp:
        return fp.read()


def version_read(rel_path):
    version_semantic_fallback = "0.1.0"
    for line in read_relative(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError(f"ERROR: Unable to parse version string from {rel_path}. "
                           f"Using fallback of {version_semantic_fallback}...")
